package uzone

import (
	"context"
	"sync"

	"github.com/google/uuid"
	"github.com/loveuer/uzone/pkg/api"
	"github.com/loveuer/uzone/pkg/log"
)

type uzone_logger struct {
	ctx context.Context
}

var uzone_logger_pool = &sync.Pool{
	New: func() any {
		return &uzone_logger{}
	},
}

func (ul *uzone_logger) traceId() string {
	if ul.ctx == nil {
		return uuid.Must(uuid.NewV7()).String()
	}

	if tid, ok := ul.ctx.Value(api.TraceKey).(string); ok && tid != "" {
		return tid
	}

	return uuid.Must(uuid.NewV7()).String()
}

func (ul *uzone_logger) Debug(msg string, data ...any) {
	log.Debug(ul.traceId()+" | "+msg, data...)

	uzone_logger_pool.Put(ul)
}

func (ul *uzone_logger) Info(msg string, data ...any) {
	log.Info(ul.traceId()+" | "+msg, data...)

	uzone_logger_pool.Put(ul)
}

func (ul *uzone_logger) Warn(msg string, data ...any) {
	log.Warn(ul.traceId()+" | "+msg, data...)

	uzone_logger_pool.Put(ul)
}

func (ul *uzone_logger) Error(msg string, data ...any) {
	log.Error(ul.traceId()+" | "+msg, data...)

	uzone_logger_pool.Put(ul)
}

func (ul *uzone_logger) Panic(msg string, data ...any) {
	log.Panic(ul.traceId()+" | "+msg, data...)
	uzone_logger_pool.Put(ul)
}

func (ul *uzone_logger) Fatal(msg string, data ...any) {
	log.Fatal(ul.traceId()+" | "+msg, data...)

	uzone_logger_pool.Put(ul)
}
